# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from Browser.generated import playwright_pb2 as playwright__pb2


class PlaywrightStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.AddCookie = channel.unary_unary(
                '/Playwright/AddCookie',
                request_serializer=playwright__pb2.Request.Json.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.GetCookies = channel.unary_unary(
                '/Playwright/GetCookies',
                request_serializer=playwright__pb2.Request.Empty.SerializeToString,
                response_deserializer=playwright__pb2.Response.Json.FromString,
                )
        self.DeleteAllCookies = channel.unary_unary(
                '/Playwright/DeleteAllCookies',
                request_serializer=playwright__pb2.Request.Empty.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.TakeScreenshot = channel.unary_unary(
                '/Playwright/TakeScreenshot',
                request_serializer=playwright__pb2.Request.ScreenshotOptions.SerializeToString,
                response_deserializer=playwright__pb2.Response.String.FromString,
                )
        self.GoTo = channel.unary_unary(
                '/Playwright/GoTo',
                request_serializer=playwright__pb2.Request.Url.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.GoBack = channel.unary_unary(
                '/Playwright/GoBack',
                request_serializer=playwright__pb2.Request.Url.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.GoForward = channel.unary_unary(
                '/Playwright/GoForward',
                request_serializer=playwright__pb2.Request.Url.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.GetTitle = channel.unary_unary(
                '/Playwright/GetTitle',
                request_serializer=playwright__pb2.Request.Empty.SerializeToString,
                response_deserializer=playwright__pb2.Response.String.FromString,
                )
        self.GetElementCount = channel.unary_unary(
                '/Playwright/GetElementCount',
                request_serializer=playwright__pb2.Request.ElementSelector.SerializeToString,
                response_deserializer=playwright__pb2.Response.Int.FromString,
                )
        self.InputText = channel.unary_unary(
                '/Playwright/InputText',
                request_serializer=playwright__pb2.Request.TextInput.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.TypeText = channel.unary_unary(
                '/Playwright/TypeText',
                request_serializer=playwright__pb2.Request.TypeText.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.FillText = channel.unary_unary(
                '/Playwright/FillText',
                request_serializer=playwright__pb2.Request.FillText.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.ClearText = channel.unary_unary(
                '/Playwright/ClearText',
                request_serializer=playwright__pb2.Request.ClearText.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.GetDomProperty = channel.unary_unary(
                '/Playwright/GetDomProperty',
                request_serializer=playwright__pb2.Request.ElementProperty.SerializeToString,
                response_deserializer=playwright__pb2.Response.String.FromString,
                )
        self.GetElementAttribute = channel.unary_unary(
                '/Playwright/GetElementAttribute',
                request_serializer=playwright__pb2.Request.ElementProperty.SerializeToString,
                response_deserializer=playwright__pb2.Response.String.FromString,
                )
        self.GetBoolProperty = channel.unary_unary(
                '/Playwright/GetBoolProperty',
                request_serializer=playwright__pb2.Request.ElementProperty.SerializeToString,
                response_deserializer=playwright__pb2.Response.Bool.FromString,
                )
        self.GetViewportSize = channel.unary_unary(
                '/Playwright/GetViewportSize',
                request_serializer=playwright__pb2.Request.Empty.SerializeToString,
                response_deserializer=playwright__pb2.Response.Json.FromString,
                )
        self.GetUrl = channel.unary_unary(
                '/Playwright/GetUrl',
                request_serializer=playwright__pb2.Request.Empty.SerializeToString,
                response_deserializer=playwright__pb2.Response.String.FromString,
                )
        self.GetPageSource = channel.unary_unary(
                '/Playwright/GetPageSource',
                request_serializer=playwright__pb2.Request.Empty.SerializeToString,
                response_deserializer=playwright__pb2.Response.String.FromString,
                )
        self.Press = channel.unary_unary(
                '/Playwright/Press',
                request_serializer=playwright__pb2.Request.PressKeys.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.GetSelectContent = channel.unary_unary(
                '/Playwright/GetSelectContent',
                request_serializer=playwright__pb2.Request.ElementSelector.SerializeToString,
                response_deserializer=playwright__pb2.Response.Select.FromString,
                )
        self.SelectOption = channel.unary_unary(
                '/Playwright/SelectOption',
                request_serializer=playwright__pb2.Request.SelectElementSelector.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.DeselectOption = channel.unary_unary(
                '/Playwright/DeselectOption',
                request_serializer=playwright__pb2.Request.ElementSelector.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.CheckCheckbox = channel.unary_unary(
                '/Playwright/CheckCheckbox',
                request_serializer=playwright__pb2.Request.ElementSelector.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.UncheckCheckbox = channel.unary_unary(
                '/Playwright/UncheckCheckbox',
                request_serializer=playwright__pb2.Request.ElementSelector.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.Health = channel.unary_unary(
                '/Playwright/Health',
                request_serializer=playwright__pb2.Request.Empty.SerializeToString,
                response_deserializer=playwright__pb2.Response.String.FromString,
                )
        self.GetElement = channel.unary_unary(
                '/Playwright/GetElement',
                request_serializer=playwright__pb2.Request.ElementSelector.SerializeToString,
                response_deserializer=playwright__pb2.Response.String.FromString,
                )
        self.GetElements = channel.unary_unary(
                '/Playwright/GetElements',
                request_serializer=playwright__pb2.Request.ElementSelector.SerializeToString,
                response_deserializer=playwright__pb2.Response.Json.FromString,
                )
        self.SetTimeout = channel.unary_unary(
                '/Playwright/SetTimeout',
                request_serializer=playwright__pb2.Request.Timeout.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.AddStyleTag = channel.unary_unary(
                '/Playwright/AddStyleTag',
                request_serializer=playwright__pb2.Request.StyleTag.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.HighlightElements = channel.unary_unary(
                '/Playwright/HighlightElements',
                request_serializer=playwright__pb2.Request.ElementSelectorWithDuration.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.Download = channel.unary_unary(
                '/Playwright/Download',
                request_serializer=playwright__pb2.Request.Url.SerializeToString,
                response_deserializer=playwright__pb2.Response.String.FromString,
                )
        self.Click = channel.unary_unary(
                '/Playwright/Click',
                request_serializer=playwright__pb2.Request.ElementSelectorWithOptions.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.Hover = channel.unary_unary(
                '/Playwright/Hover',
                request_serializer=playwright__pb2.Request.ElementSelectorWithOptions.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.Focus = channel.unary_unary(
                '/Playwright/Focus',
                request_serializer=playwright__pb2.Request.ElementSelector.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.WaitForElementsState = channel.unary_unary(
                '/Playwright/WaitForElementsState',
                request_serializer=playwright__pb2.Request.ElementSelectorWithOptions.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.WaitForFunction = channel.unary_unary(
                '/Playwright/WaitForFunction',
                request_serializer=playwright__pb2.Request.WaitForFunctionOptions.SerializeToString,
                response_deserializer=playwright__pb2.Response.Json.FromString,
                )
        self.ExecuteJavascript = channel.unary_unary(
                '/Playwright/ExecuteJavascript',
                request_serializer=playwright__pb2.Request.JavascriptCode.SerializeToString,
                response_deserializer=playwright__pb2.Response.JavascriptExecutionResult.FromString,
                )
        self.GetPageState = channel.unary_unary(
                '/Playwright/GetPageState',
                request_serializer=playwright__pb2.Request.Empty.SerializeToString,
                response_deserializer=playwright__pb2.Response.JavascriptExecutionResult.FromString,
                )
        self.SetViewportSize = channel.unary_unary(
                '/Playwright/SetViewportSize',
                request_serializer=playwright__pb2.Request.Viewport.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.GetStyle = channel.unary_unary(
                '/Playwright/GetStyle',
                request_serializer=playwright__pb2.Request.ElementSelector.SerializeToString,
                response_deserializer=playwright__pb2.Response.Json.FromString,
                )
        self.GetBoundingBox = channel.unary_unary(
                '/Playwright/GetBoundingBox',
                request_serializer=playwright__pb2.Request.ElementSelector.SerializeToString,
                response_deserializer=playwright__pb2.Response.Json.FromString,
                )
        self.HttpRequest = channel.unary_unary(
                '/Playwright/HttpRequest',
                request_serializer=playwright__pb2.Request.HttpRequest.SerializeToString,
                response_deserializer=playwright__pb2.Response.Json.FromString,
                )
        self.WaitForRequest = channel.unary_unary(
                '/Playwright/WaitForRequest',
                request_serializer=playwright__pb2.Request.HttpCapture.SerializeToString,
                response_deserializer=playwright__pb2.Response.String.FromString,
                )
        self.WaitForResponse = channel.unary_unary(
                '/Playwright/WaitForResponse',
                request_serializer=playwright__pb2.Request.HttpCapture.SerializeToString,
                response_deserializer=playwright__pb2.Response.Json.FromString,
                )
        self.WaitForDownload = channel.unary_unary(
                '/Playwright/WaitForDownload',
                request_serializer=playwright__pb2.Request.FilePath.SerializeToString,
                response_deserializer=playwright__pb2.Response.Json.FromString,
                )
        self.WaitUntilNetworkIsIdle = channel.unary_unary(
                '/Playwright/WaitUntilNetworkIsIdle',
                request_serializer=playwright__pb2.Request.Timeout.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.SetGeolocation = channel.unary_unary(
                '/Playwright/SetGeolocation',
                request_serializer=playwright__pb2.Request.Json.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.GetDevice = channel.unary_unary(
                '/Playwright/GetDevice',
                request_serializer=playwright__pb2.Request.Device.SerializeToString,
                response_deserializer=playwright__pb2.Response.Json.FromString,
                )
        self.GetDevices = channel.unary_unary(
                '/Playwright/GetDevices',
                request_serializer=playwright__pb2.Request.Empty.SerializeToString,
                response_deserializer=playwright__pb2.Response.Json.FromString,
                )
        self.HandleAlert = channel.unary_unary(
                '/Playwright/HandleAlert',
                request_serializer=playwright__pb2.Request.AlertAction.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.MouseButton = channel.unary_unary(
                '/Playwright/MouseButton',
                request_serializer=playwright__pb2.Request.MouseButtonOptions.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.MouseMove = channel.unary_unary(
                '/Playwright/MouseMove',
                request_serializer=playwright__pb2.Request.Json.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.KeyboardKey = channel.unary_unary(
                '/Playwright/KeyboardKey',
                request_serializer=playwright__pb2.Request.KeyboardKeypress.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.KeyboardInput = channel.unary_unary(
                '/Playwright/KeyboardInput',
                request_serializer=playwright__pb2.Request.KeyboardInputOptions.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.UploadFile = channel.unary_unary(
                '/Playwright/UploadFile',
                request_serializer=playwright__pb2.Request.FilePath.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.SetOffline = channel.unary_unary(
                '/Playwright/SetOffline',
                request_serializer=playwright__pb2.Request.Bool.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.Reload = channel.unary_unary(
                '/Playwright/Reload',
                request_serializer=playwright__pb2.Request.Empty.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.SwitchPage = channel.unary_unary(
                '/Playwright/SwitchPage',
                request_serializer=playwright__pb2.Request.IdWithTimeout.SerializeToString,
                response_deserializer=playwright__pb2.Response.String.FromString,
                )
        self.SwitchContext = channel.unary_unary(
                '/Playwright/SwitchContext',
                request_serializer=playwright__pb2.Request.Index.SerializeToString,
                response_deserializer=playwright__pb2.Response.String.FromString,
                )
        self.SwitchBrowser = channel.unary_unary(
                '/Playwright/SwitchBrowser',
                request_serializer=playwright__pb2.Request.Index.SerializeToString,
                response_deserializer=playwright__pb2.Response.String.FromString,
                )
        self.NewPage = channel.unary_unary(
                '/Playwright/NewPage',
                request_serializer=playwright__pb2.Request.Url.SerializeToString,
                response_deserializer=playwright__pb2.Response.String.FromString,
                )
        self.NewContext = channel.unary_unary(
                '/Playwright/NewContext',
                request_serializer=playwright__pb2.Request.Context.SerializeToString,
                response_deserializer=playwright__pb2.Response.String.FromString,
                )
        self.NewBrowser = channel.unary_unary(
                '/Playwright/NewBrowser',
                request_serializer=playwright__pb2.Request.Browser.SerializeToString,
                response_deserializer=playwright__pb2.Response.String.FromString,
                )
        self.CloseBrowser = channel.unary_unary(
                '/Playwright/CloseBrowser',
                request_serializer=playwright__pb2.Request.Empty.SerializeToString,
                response_deserializer=playwright__pb2.Response.String.FromString,
                )
        self.CloseAllBrowsers = channel.unary_unary(
                '/Playwright/CloseAllBrowsers',
                request_serializer=playwright__pb2.Request.Empty.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.CloseContext = channel.unary_unary(
                '/Playwright/CloseContext',
                request_serializer=playwright__pb2.Request.Empty.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.ClosePage = channel.unary_unary(
                '/Playwright/ClosePage',
                request_serializer=playwright__pb2.Request.Empty.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.GetBrowserCatalog = channel.unary_unary(
                '/Playwright/GetBrowserCatalog',
                request_serializer=playwright__pb2.Request.Empty.SerializeToString,
                response_deserializer=playwright__pb2.Response.Json.FromString,
                )


class PlaywrightServicer(object):
    """Missing associated documentation comment in .proto file."""

    def AddCookie(self, request, context):
        """Cookie messages 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetCookies(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteAllCookies(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def TakeScreenshot(self, request, context):
        """Screen shot method 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GoTo(self, request, context):
        """Opens the url in currently open Playwright page 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GoBack(self, request, context):
        """Navigate to the next page in history 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GoForward(self, request, context):
        """Navigate to the previous page in history. 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTitle(self, request, context):
        """Gets title of currently open Playwright page 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetElementCount(self, request, context):
        """Returns the count of elements found with selector 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def InputText(self, request, context):
        """Wraps playwrights page.fill to input text into input specified with selector 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def TypeText(self, request, context):
        """Wraps playwrights page.type to type text into input specified with selector 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def FillText(self, request, context):
        """Wraps playwrights page.fill to fill text of input specified with selector 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ClearText(self, request, context):
        """Wraps playwrights page.fill with empty text to clear input specified with selector 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetDomProperty(self, request, context):
        """Gets the DOM property 'property' of selector specified element 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetElementAttribute(self, request, context):
        """Gets the XML attribute 'attribute' of selector specified element 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetBoolProperty(self, request, context):
        """Gets the boolean DOM property 'property' of selector specified element 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetViewportSize(self, request, context):
        """Wraps playwrights page.textContent, returns textcontent of element by selector 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetUrl(self, request, context):
        """Returns current playwright page url
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPageSource(self, request, context):
        """Gets pages HTML code 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Press(self, request, context):
        """Inputs a list of keypresses to element specified by selector 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetSelectContent(self, request, context):
        """Gets the Select element specified by selector and returns the contents 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SelectOption(self, request, context):
        """Selects option matching matcher in Select element matching selector 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeselectOption(self, request, context):
        """Deselects options in Select element matching selector 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CheckCheckbox(self, request, context):
        """Checks checkbox specified by selector 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UncheckCheckbox(self, request, context):
        """Unchecks checkbox specified by selector 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Health(self, request, context):
        """Health check endpoint for the service 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetElement(self, request, context):
        """Returns a reference to a Playwirght element handle.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetElements(self, request, context):
        """Returns references to multiple Playwirght element handles.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetTimeout(self, request, context):
        """Set's  playwright timeout 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AddStyleTag(self, request, context):
        """Adds a <style> to head of site. 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def HighlightElements(self, request, context):
        """Highlights elements matching selector for duration 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Download(self, request, context):
        """Download url content 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Click(self, request, context):
        """Clicks element specified by selector and options 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Hover(self, request, context):
        """Hovers an element specified by selector and options 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Focus(self, request, context):
        """Focuses element specified by selector 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WaitForElementsState(self, request, context):
        """Waits for element be in a specific state
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WaitForFunction(self, request, context):
        """Executes javascript on the active page and retries until the expression is true 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ExecuteJavascript(self, request, context):
        """Executes javascript on the active page 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPageState(self, request, context):
        """Get Page State JSON 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetViewportSize(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetStyle(self, request, context):
        """Gets an elements computed style 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetBoundingBox(self, request, context):
        """Gets elements x, y coordinates and width, height as json object 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def HttpRequest(self, request, context):
        """Makes a `fetch` request in the browser 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WaitForRequest(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WaitForResponse(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WaitForDownload(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WaitUntilNetworkIsIdle(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetGeolocation(self, request, context):
        """Geolocation 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetDevice(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetDevices(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def HandleAlert(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def MouseButton(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def MouseMove(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def KeyboardKey(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def KeyboardInput(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UploadFile(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetOffline(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Reload(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SwitchPage(self, request, context):
        """Playwright State handling keywords 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SwitchContext(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SwitchBrowser(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def NewPage(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def NewContext(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def NewBrowser(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CloseBrowser(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CloseAllBrowsers(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CloseContext(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ClosePage(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetBrowserCatalog(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_PlaywrightServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'AddCookie': grpc.unary_unary_rpc_method_handler(
                    servicer.AddCookie,
                    request_deserializer=playwright__pb2.Request.Json.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'GetCookies': grpc.unary_unary_rpc_method_handler(
                    servicer.GetCookies,
                    request_deserializer=playwright__pb2.Request.Empty.FromString,
                    response_serializer=playwright__pb2.Response.Json.SerializeToString,
            ),
            'DeleteAllCookies': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteAllCookies,
                    request_deserializer=playwright__pb2.Request.Empty.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'TakeScreenshot': grpc.unary_unary_rpc_method_handler(
                    servicer.TakeScreenshot,
                    request_deserializer=playwright__pb2.Request.ScreenshotOptions.FromString,
                    response_serializer=playwright__pb2.Response.String.SerializeToString,
            ),
            'GoTo': grpc.unary_unary_rpc_method_handler(
                    servicer.GoTo,
                    request_deserializer=playwright__pb2.Request.Url.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'GoBack': grpc.unary_unary_rpc_method_handler(
                    servicer.GoBack,
                    request_deserializer=playwright__pb2.Request.Url.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'GoForward': grpc.unary_unary_rpc_method_handler(
                    servicer.GoForward,
                    request_deserializer=playwright__pb2.Request.Url.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'GetTitle': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTitle,
                    request_deserializer=playwright__pb2.Request.Empty.FromString,
                    response_serializer=playwright__pb2.Response.String.SerializeToString,
            ),
            'GetElementCount': grpc.unary_unary_rpc_method_handler(
                    servicer.GetElementCount,
                    request_deserializer=playwright__pb2.Request.ElementSelector.FromString,
                    response_serializer=playwright__pb2.Response.Int.SerializeToString,
            ),
            'InputText': grpc.unary_unary_rpc_method_handler(
                    servicer.InputText,
                    request_deserializer=playwright__pb2.Request.TextInput.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'TypeText': grpc.unary_unary_rpc_method_handler(
                    servicer.TypeText,
                    request_deserializer=playwright__pb2.Request.TypeText.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'FillText': grpc.unary_unary_rpc_method_handler(
                    servicer.FillText,
                    request_deserializer=playwright__pb2.Request.FillText.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'ClearText': grpc.unary_unary_rpc_method_handler(
                    servicer.ClearText,
                    request_deserializer=playwright__pb2.Request.ClearText.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'GetDomProperty': grpc.unary_unary_rpc_method_handler(
                    servicer.GetDomProperty,
                    request_deserializer=playwright__pb2.Request.ElementProperty.FromString,
                    response_serializer=playwright__pb2.Response.String.SerializeToString,
            ),
            'GetElementAttribute': grpc.unary_unary_rpc_method_handler(
                    servicer.GetElementAttribute,
                    request_deserializer=playwright__pb2.Request.ElementProperty.FromString,
                    response_serializer=playwright__pb2.Response.String.SerializeToString,
            ),
            'GetBoolProperty': grpc.unary_unary_rpc_method_handler(
                    servicer.GetBoolProperty,
                    request_deserializer=playwright__pb2.Request.ElementProperty.FromString,
                    response_serializer=playwright__pb2.Response.Bool.SerializeToString,
            ),
            'GetViewportSize': grpc.unary_unary_rpc_method_handler(
                    servicer.GetViewportSize,
                    request_deserializer=playwright__pb2.Request.Empty.FromString,
                    response_serializer=playwright__pb2.Response.Json.SerializeToString,
            ),
            'GetUrl': grpc.unary_unary_rpc_method_handler(
                    servicer.GetUrl,
                    request_deserializer=playwright__pb2.Request.Empty.FromString,
                    response_serializer=playwright__pb2.Response.String.SerializeToString,
            ),
            'GetPageSource': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPageSource,
                    request_deserializer=playwright__pb2.Request.Empty.FromString,
                    response_serializer=playwright__pb2.Response.String.SerializeToString,
            ),
            'Press': grpc.unary_unary_rpc_method_handler(
                    servicer.Press,
                    request_deserializer=playwright__pb2.Request.PressKeys.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'GetSelectContent': grpc.unary_unary_rpc_method_handler(
                    servicer.GetSelectContent,
                    request_deserializer=playwright__pb2.Request.ElementSelector.FromString,
                    response_serializer=playwright__pb2.Response.Select.SerializeToString,
            ),
            'SelectOption': grpc.unary_unary_rpc_method_handler(
                    servicer.SelectOption,
                    request_deserializer=playwright__pb2.Request.SelectElementSelector.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'DeselectOption': grpc.unary_unary_rpc_method_handler(
                    servicer.DeselectOption,
                    request_deserializer=playwright__pb2.Request.ElementSelector.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'CheckCheckbox': grpc.unary_unary_rpc_method_handler(
                    servicer.CheckCheckbox,
                    request_deserializer=playwright__pb2.Request.ElementSelector.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'UncheckCheckbox': grpc.unary_unary_rpc_method_handler(
                    servicer.UncheckCheckbox,
                    request_deserializer=playwright__pb2.Request.ElementSelector.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'Health': grpc.unary_unary_rpc_method_handler(
                    servicer.Health,
                    request_deserializer=playwright__pb2.Request.Empty.FromString,
                    response_serializer=playwright__pb2.Response.String.SerializeToString,
            ),
            'GetElement': grpc.unary_unary_rpc_method_handler(
                    servicer.GetElement,
                    request_deserializer=playwright__pb2.Request.ElementSelector.FromString,
                    response_serializer=playwright__pb2.Response.String.SerializeToString,
            ),
            'GetElements': grpc.unary_unary_rpc_method_handler(
                    servicer.GetElements,
                    request_deserializer=playwright__pb2.Request.ElementSelector.FromString,
                    response_serializer=playwright__pb2.Response.Json.SerializeToString,
            ),
            'SetTimeout': grpc.unary_unary_rpc_method_handler(
                    servicer.SetTimeout,
                    request_deserializer=playwright__pb2.Request.Timeout.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'AddStyleTag': grpc.unary_unary_rpc_method_handler(
                    servicer.AddStyleTag,
                    request_deserializer=playwright__pb2.Request.StyleTag.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'HighlightElements': grpc.unary_unary_rpc_method_handler(
                    servicer.HighlightElements,
                    request_deserializer=playwright__pb2.Request.ElementSelectorWithDuration.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'Download': grpc.unary_unary_rpc_method_handler(
                    servicer.Download,
                    request_deserializer=playwright__pb2.Request.Url.FromString,
                    response_serializer=playwright__pb2.Response.String.SerializeToString,
            ),
            'Click': grpc.unary_unary_rpc_method_handler(
                    servicer.Click,
                    request_deserializer=playwright__pb2.Request.ElementSelectorWithOptions.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'Hover': grpc.unary_unary_rpc_method_handler(
                    servicer.Hover,
                    request_deserializer=playwright__pb2.Request.ElementSelectorWithOptions.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'Focus': grpc.unary_unary_rpc_method_handler(
                    servicer.Focus,
                    request_deserializer=playwright__pb2.Request.ElementSelector.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'WaitForElementsState': grpc.unary_unary_rpc_method_handler(
                    servicer.WaitForElementsState,
                    request_deserializer=playwright__pb2.Request.ElementSelectorWithOptions.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'WaitForFunction': grpc.unary_unary_rpc_method_handler(
                    servicer.WaitForFunction,
                    request_deserializer=playwright__pb2.Request.WaitForFunctionOptions.FromString,
                    response_serializer=playwright__pb2.Response.Json.SerializeToString,
            ),
            'ExecuteJavascript': grpc.unary_unary_rpc_method_handler(
                    servicer.ExecuteJavascript,
                    request_deserializer=playwright__pb2.Request.JavascriptCode.FromString,
                    response_serializer=playwright__pb2.Response.JavascriptExecutionResult.SerializeToString,
            ),
            'GetPageState': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPageState,
                    request_deserializer=playwright__pb2.Request.Empty.FromString,
                    response_serializer=playwright__pb2.Response.JavascriptExecutionResult.SerializeToString,
            ),
            'SetViewportSize': grpc.unary_unary_rpc_method_handler(
                    servicer.SetViewportSize,
                    request_deserializer=playwright__pb2.Request.Viewport.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'GetStyle': grpc.unary_unary_rpc_method_handler(
                    servicer.GetStyle,
                    request_deserializer=playwright__pb2.Request.ElementSelector.FromString,
                    response_serializer=playwright__pb2.Response.Json.SerializeToString,
            ),
            'GetBoundingBox': grpc.unary_unary_rpc_method_handler(
                    servicer.GetBoundingBox,
                    request_deserializer=playwright__pb2.Request.ElementSelector.FromString,
                    response_serializer=playwright__pb2.Response.Json.SerializeToString,
            ),
            'HttpRequest': grpc.unary_unary_rpc_method_handler(
                    servicer.HttpRequest,
                    request_deserializer=playwright__pb2.Request.HttpRequest.FromString,
                    response_serializer=playwright__pb2.Response.Json.SerializeToString,
            ),
            'WaitForRequest': grpc.unary_unary_rpc_method_handler(
                    servicer.WaitForRequest,
                    request_deserializer=playwright__pb2.Request.HttpCapture.FromString,
                    response_serializer=playwright__pb2.Response.String.SerializeToString,
            ),
            'WaitForResponse': grpc.unary_unary_rpc_method_handler(
                    servicer.WaitForResponse,
                    request_deserializer=playwright__pb2.Request.HttpCapture.FromString,
                    response_serializer=playwright__pb2.Response.Json.SerializeToString,
            ),
            'WaitForDownload': grpc.unary_unary_rpc_method_handler(
                    servicer.WaitForDownload,
                    request_deserializer=playwright__pb2.Request.FilePath.FromString,
                    response_serializer=playwright__pb2.Response.Json.SerializeToString,
            ),
            'WaitUntilNetworkIsIdle': grpc.unary_unary_rpc_method_handler(
                    servicer.WaitUntilNetworkIsIdle,
                    request_deserializer=playwright__pb2.Request.Timeout.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'SetGeolocation': grpc.unary_unary_rpc_method_handler(
                    servicer.SetGeolocation,
                    request_deserializer=playwright__pb2.Request.Json.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'GetDevice': grpc.unary_unary_rpc_method_handler(
                    servicer.GetDevice,
                    request_deserializer=playwright__pb2.Request.Device.FromString,
                    response_serializer=playwright__pb2.Response.Json.SerializeToString,
            ),
            'GetDevices': grpc.unary_unary_rpc_method_handler(
                    servicer.GetDevices,
                    request_deserializer=playwright__pb2.Request.Empty.FromString,
                    response_serializer=playwright__pb2.Response.Json.SerializeToString,
            ),
            'HandleAlert': grpc.unary_unary_rpc_method_handler(
                    servicer.HandleAlert,
                    request_deserializer=playwright__pb2.Request.AlertAction.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'MouseButton': grpc.unary_unary_rpc_method_handler(
                    servicer.MouseButton,
                    request_deserializer=playwright__pb2.Request.MouseButtonOptions.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'MouseMove': grpc.unary_unary_rpc_method_handler(
                    servicer.MouseMove,
                    request_deserializer=playwright__pb2.Request.Json.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'KeyboardKey': grpc.unary_unary_rpc_method_handler(
                    servicer.KeyboardKey,
                    request_deserializer=playwright__pb2.Request.KeyboardKeypress.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'KeyboardInput': grpc.unary_unary_rpc_method_handler(
                    servicer.KeyboardInput,
                    request_deserializer=playwright__pb2.Request.KeyboardInputOptions.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'UploadFile': grpc.unary_unary_rpc_method_handler(
                    servicer.UploadFile,
                    request_deserializer=playwright__pb2.Request.FilePath.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'SetOffline': grpc.unary_unary_rpc_method_handler(
                    servicer.SetOffline,
                    request_deserializer=playwright__pb2.Request.Bool.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'Reload': grpc.unary_unary_rpc_method_handler(
                    servicer.Reload,
                    request_deserializer=playwright__pb2.Request.Empty.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'SwitchPage': grpc.unary_unary_rpc_method_handler(
                    servicer.SwitchPage,
                    request_deserializer=playwright__pb2.Request.IdWithTimeout.FromString,
                    response_serializer=playwright__pb2.Response.String.SerializeToString,
            ),
            'SwitchContext': grpc.unary_unary_rpc_method_handler(
                    servicer.SwitchContext,
                    request_deserializer=playwright__pb2.Request.Index.FromString,
                    response_serializer=playwright__pb2.Response.String.SerializeToString,
            ),
            'SwitchBrowser': grpc.unary_unary_rpc_method_handler(
                    servicer.SwitchBrowser,
                    request_deserializer=playwright__pb2.Request.Index.FromString,
                    response_serializer=playwright__pb2.Response.String.SerializeToString,
            ),
            'NewPage': grpc.unary_unary_rpc_method_handler(
                    servicer.NewPage,
                    request_deserializer=playwright__pb2.Request.Url.FromString,
                    response_serializer=playwright__pb2.Response.String.SerializeToString,
            ),
            'NewContext': grpc.unary_unary_rpc_method_handler(
                    servicer.NewContext,
                    request_deserializer=playwright__pb2.Request.Context.FromString,
                    response_serializer=playwright__pb2.Response.String.SerializeToString,
            ),
            'NewBrowser': grpc.unary_unary_rpc_method_handler(
                    servicer.NewBrowser,
                    request_deserializer=playwright__pb2.Request.Browser.FromString,
                    response_serializer=playwright__pb2.Response.String.SerializeToString,
            ),
            'CloseBrowser': grpc.unary_unary_rpc_method_handler(
                    servicer.CloseBrowser,
                    request_deserializer=playwright__pb2.Request.Empty.FromString,
                    response_serializer=playwright__pb2.Response.String.SerializeToString,
            ),
            'CloseAllBrowsers': grpc.unary_unary_rpc_method_handler(
                    servicer.CloseAllBrowsers,
                    request_deserializer=playwright__pb2.Request.Empty.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'CloseContext': grpc.unary_unary_rpc_method_handler(
                    servicer.CloseContext,
                    request_deserializer=playwright__pb2.Request.Empty.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'ClosePage': grpc.unary_unary_rpc_method_handler(
                    servicer.ClosePage,
                    request_deserializer=playwright__pb2.Request.Empty.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'GetBrowserCatalog': grpc.unary_unary_rpc_method_handler(
                    servicer.GetBrowserCatalog,
                    request_deserializer=playwright__pb2.Request.Empty.FromString,
                    response_serializer=playwright__pb2.Response.Json.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'Playwright', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Playwright(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def AddCookie(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/AddCookie',
            playwright__pb2.Request.Json.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetCookies(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GetCookies',
            playwright__pb2.Request.Empty.SerializeToString,
            playwright__pb2.Response.Json.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteAllCookies(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/DeleteAllCookies',
            playwright__pb2.Request.Empty.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def TakeScreenshot(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/TakeScreenshot',
            playwright__pb2.Request.ScreenshotOptions.SerializeToString,
            playwright__pb2.Response.String.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GoTo(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GoTo',
            playwright__pb2.Request.Url.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GoBack(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GoBack',
            playwright__pb2.Request.Url.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GoForward(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GoForward',
            playwright__pb2.Request.Url.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTitle(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GetTitle',
            playwright__pb2.Request.Empty.SerializeToString,
            playwright__pb2.Response.String.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetElementCount(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GetElementCount',
            playwright__pb2.Request.ElementSelector.SerializeToString,
            playwright__pb2.Response.Int.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def InputText(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/InputText',
            playwright__pb2.Request.TextInput.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def TypeText(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/TypeText',
            playwright__pb2.Request.TypeText.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def FillText(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/FillText',
            playwright__pb2.Request.FillText.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ClearText(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/ClearText',
            playwright__pb2.Request.ClearText.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetDomProperty(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GetDomProperty',
            playwright__pb2.Request.ElementProperty.SerializeToString,
            playwright__pb2.Response.String.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetElementAttribute(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GetElementAttribute',
            playwright__pb2.Request.ElementProperty.SerializeToString,
            playwright__pb2.Response.String.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetBoolProperty(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GetBoolProperty',
            playwright__pb2.Request.ElementProperty.SerializeToString,
            playwright__pb2.Response.Bool.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetViewportSize(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GetViewportSize',
            playwright__pb2.Request.Empty.SerializeToString,
            playwright__pb2.Response.Json.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetUrl(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GetUrl',
            playwright__pb2.Request.Empty.SerializeToString,
            playwright__pb2.Response.String.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetPageSource(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GetPageSource',
            playwright__pb2.Request.Empty.SerializeToString,
            playwright__pb2.Response.String.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Press(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/Press',
            playwright__pb2.Request.PressKeys.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetSelectContent(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GetSelectContent',
            playwright__pb2.Request.ElementSelector.SerializeToString,
            playwright__pb2.Response.Select.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SelectOption(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/SelectOption',
            playwright__pb2.Request.SelectElementSelector.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeselectOption(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/DeselectOption',
            playwright__pb2.Request.ElementSelector.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CheckCheckbox(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/CheckCheckbox',
            playwright__pb2.Request.ElementSelector.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UncheckCheckbox(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/UncheckCheckbox',
            playwright__pb2.Request.ElementSelector.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Health(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/Health',
            playwright__pb2.Request.Empty.SerializeToString,
            playwright__pb2.Response.String.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetElement(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GetElement',
            playwright__pb2.Request.ElementSelector.SerializeToString,
            playwright__pb2.Response.String.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetElements(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GetElements',
            playwright__pb2.Request.ElementSelector.SerializeToString,
            playwright__pb2.Response.Json.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetTimeout(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/SetTimeout',
            playwright__pb2.Request.Timeout.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AddStyleTag(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/AddStyleTag',
            playwright__pb2.Request.StyleTag.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def HighlightElements(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/HighlightElements',
            playwright__pb2.Request.ElementSelectorWithDuration.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Download(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/Download',
            playwright__pb2.Request.Url.SerializeToString,
            playwright__pb2.Response.String.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Click(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/Click',
            playwright__pb2.Request.ElementSelectorWithOptions.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Hover(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/Hover',
            playwright__pb2.Request.ElementSelectorWithOptions.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Focus(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/Focus',
            playwright__pb2.Request.ElementSelector.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WaitForElementsState(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/WaitForElementsState',
            playwright__pb2.Request.ElementSelectorWithOptions.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WaitForFunction(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/WaitForFunction',
            playwright__pb2.Request.WaitForFunctionOptions.SerializeToString,
            playwright__pb2.Response.Json.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ExecuteJavascript(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/ExecuteJavascript',
            playwright__pb2.Request.JavascriptCode.SerializeToString,
            playwright__pb2.Response.JavascriptExecutionResult.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetPageState(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GetPageState',
            playwright__pb2.Request.Empty.SerializeToString,
            playwright__pb2.Response.JavascriptExecutionResult.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetViewportSize(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/SetViewportSize',
            playwright__pb2.Request.Viewport.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetStyle(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GetStyle',
            playwright__pb2.Request.ElementSelector.SerializeToString,
            playwright__pb2.Response.Json.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetBoundingBox(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GetBoundingBox',
            playwright__pb2.Request.ElementSelector.SerializeToString,
            playwright__pb2.Response.Json.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def HttpRequest(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/HttpRequest',
            playwright__pb2.Request.HttpRequest.SerializeToString,
            playwright__pb2.Response.Json.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WaitForRequest(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/WaitForRequest',
            playwright__pb2.Request.HttpCapture.SerializeToString,
            playwright__pb2.Response.String.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WaitForResponse(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/WaitForResponse',
            playwright__pb2.Request.HttpCapture.SerializeToString,
            playwright__pb2.Response.Json.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WaitForDownload(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/WaitForDownload',
            playwright__pb2.Request.FilePath.SerializeToString,
            playwright__pb2.Response.Json.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WaitUntilNetworkIsIdle(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/WaitUntilNetworkIsIdle',
            playwright__pb2.Request.Timeout.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetGeolocation(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/SetGeolocation',
            playwright__pb2.Request.Json.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetDevice(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GetDevice',
            playwright__pb2.Request.Device.SerializeToString,
            playwright__pb2.Response.Json.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetDevices(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GetDevices',
            playwright__pb2.Request.Empty.SerializeToString,
            playwright__pb2.Response.Json.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def HandleAlert(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/HandleAlert',
            playwright__pb2.Request.AlertAction.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def MouseButton(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/MouseButton',
            playwright__pb2.Request.MouseButtonOptions.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def MouseMove(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/MouseMove',
            playwright__pb2.Request.Json.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def KeyboardKey(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/KeyboardKey',
            playwright__pb2.Request.KeyboardKeypress.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def KeyboardInput(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/KeyboardInput',
            playwright__pb2.Request.KeyboardInputOptions.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UploadFile(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/UploadFile',
            playwright__pb2.Request.FilePath.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetOffline(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/SetOffline',
            playwright__pb2.Request.Bool.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Reload(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/Reload',
            playwright__pb2.Request.Empty.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SwitchPage(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/SwitchPage',
            playwright__pb2.Request.IdWithTimeout.SerializeToString,
            playwright__pb2.Response.String.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SwitchContext(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/SwitchContext',
            playwright__pb2.Request.Index.SerializeToString,
            playwright__pb2.Response.String.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SwitchBrowser(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/SwitchBrowser',
            playwright__pb2.Request.Index.SerializeToString,
            playwright__pb2.Response.String.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def NewPage(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/NewPage',
            playwright__pb2.Request.Url.SerializeToString,
            playwright__pb2.Response.String.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def NewContext(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/NewContext',
            playwright__pb2.Request.Context.SerializeToString,
            playwright__pb2.Response.String.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def NewBrowser(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/NewBrowser',
            playwright__pb2.Request.Browser.SerializeToString,
            playwright__pb2.Response.String.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CloseBrowser(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/CloseBrowser',
            playwright__pb2.Request.Empty.SerializeToString,
            playwright__pb2.Response.String.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CloseAllBrowsers(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/CloseAllBrowsers',
            playwright__pb2.Request.Empty.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CloseContext(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/CloseContext',
            playwright__pb2.Request.Empty.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ClosePage(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/ClosePage',
            playwright__pb2.Request.Empty.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetBrowserCatalog(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GetBrowserCatalog',
            playwright__pb2.Request.Empty.SerializeToString,
            playwright__pb2.Response.Json.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# Copyright 2020-     Robot Framework Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import base64
import json
from datetime import timedelta
from pathlib import Path
from typing import Optional

from robot.utils import get_link_path  # type: ignore
from robotlibcore import keyword  # type: ignore

from ..base import LibraryComponent
from ..generated.playwright_pb2 import Request
from ..utils import logger


class Control(LibraryComponent):
    """Keywords to do things on the current browser page and modify the page"""

    @keyword(tags=["Setter", "BrowserControl"])
    def go_forward(self):
        """Navigates to the next page in history."""
        with self.playwright.grpc_channel() as stub:
            response = stub.GoForward(Request.Empty())
            logger.info(response.log)

    @keyword(tags=["Setter", "BrowserControl"])
    def go_back(self):
        """Navigates to the previous page in history."""
        with self.playwright.grpc_channel() as stub:
            response = stub.GoBack(Request.Empty())
            logger.info(response.log)

    @keyword(tags=["Setter", "BrowserControl"])
    def go_to(self, url: str):
        """Navigates to the given ``url``.

        ``url`` <str> URL to be navigated to. **Required**
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.GoTo(Request().Url(url=url))
            logger.info(response.log)

    def _get_screenshot_path(self, filename: str) -> Path:
        directory = self.library.outputdir
        # Filename didn't contain {index}
        if "{index}" not in filename:
            return Path(directory) / filename
        index = 0
        while True:
            index += 1
            indexed = Path(filename.replace("{index}", str(index)))
            logger.trace(indexed)
            path = Path(directory) / indexed
            # Unique path was found
            if not path.with_suffix(".png").is_file():
                return path

    @keyword(tags=["PageContent"])
    def take_screenshot(
        self,
        filename: str = "robotframework-browser-screenshot-{index}",
        selector: str = "",
        fullPage: bool = False,
    ) -> str:
        """Takes a screenshot of the current window and saves it to ``path``. Saves it as a png.

        ``filename`` <str> Filename into which to save. The file will be saved into the robot framework output
        directory by default. String ``{index}`` in path will be replaced with a rolling number. Use this to not
        override filenames. If filename equals to EMBED (case insensitive), then screenshot is embedded as
        Base64 image to the log.html. The image is saved temporally to the disk and warning is displayed
        if removing the temporary file fails.

        ``selector`` <str> Take a screenshot of the element matched by selector.
        See the `Finding elements` section for details about the selectors.
        If not provided take a screenshot of current viewport.

        ``fullPage`` <bool> When True, takes a screenshot of the full scrollable page,
        instead of the currently visible viewport. Defaults to False.
        """
        if self._is_embed(filename):
            logger.debug("Embedding image to log.html.")
        else:
            logger.debug(f"Using {filename} to take screenshot.")
        file_path = self._take_screenshot(filename, selector, fullPage)
        if self._is_embed(filename):
            return self._embed_to_log(file_path)
        return self._log_image_link(file_path)

    def _log_image_link(self, file_path: str) -> str:
        relative_path = get_link_path(file_path, self.library.outputdir)
        logger.info(
            '</td></tr><tr><td colspan="3">'
            f'<a href="{relative_path}"><img src="{relative_path}" width="800px"></a>',
            html=True,
        )
        return file_path

    def _embed_to_log(self, file_path) -> str:
        png = Path(file_path)
        with png.open("rb") as png_file:
            encoded_string = base64.b64encode(png_file.read())
        # log statement is copied from:
        # https://github.com/robotframework/SeleniumLibrary/blob/master/src/SeleniumLibrary/keywords/screenshot.py
        logger.info(
            '</td></tr><tr><td colspan="3">'
            '<img alt="screenshot" class="robot-seleniumlibrary-screenshot" '
            f'src="data:image/png;base64,{encoded_string.decode()}" width="900px">',
            html=True,
        )
        try:
            png.unlink()
        except Exception:
            logger.warn(f"Could not remove {png}")
        return "EMBED"

    def _take_screenshot(self, filename: str, selector: str, fullPage: bool) -> str:
        string_path_no_extension = str(self._get_screenshot_path(filename))
        with self.playwright.grpc_channel() as stub:
            response = stub.TakeScreenshot(
                Request().ScreenshotOptions(
                    path=string_path_no_extension, selector=selector, fullPage=fullPage
                )
            )
        logger.debug(response.log)
        return response.body

    def _is_embed(self, filename: str) -> bool:
        return True if filename.upper() == "EMBED" else False

    @keyword(tags=["Setter", "Config"])
    def set_browser_timeout(self, timeout: timedelta) -> str:
        """Sets the timeout used by most input and getter keywords.

        ``timeout`` <str> Timeout of it is for current playwright context. **Required**

        Returns the previous value of the timeout.
        """
        old_timeout = self.millisecs_to_timestr(self.timeout)
        self.timeout = self.convert_timeout(timeout)
        with self.playwright.grpc_channel() as stub:
            response = stub.SetTimeout(Request().Timeout(timeout=self.timeout))
            logger.info(response.log)
        return old_timeout

    @keyword(tags=["Setter", "Config"])
    def set_retry_assertions_for(self, timeout: timedelta) -> str:
        """Sets the timeout used in retrying assertions when they fail.

        ``timeout`` <str>

        Returns the previous value of the retry_assertions_until.
        """
        old_retry_assertions_for = self.millisecs_to_timestr(self.retry_assertions_for)
        self.retry_assertions_for = self.convert_timeout(timeout)
        return old_retry_assertions_for

    @keyword(tags=["Setter", "BrowserControl"])
    def set_viewport_size(self, width: int, height: int):
        """Sets current Pages viewport size to specified dimensions.

        In the case of multiple pages in a single browser,
        each page can have its own viewport size. However,
        `New Context` allows to set viewport size (and more) for all
        later opened pages in the context at once.

        `Set Viewport Size` will resize the page.
        A lot of websites don't expect phones to change size,
        so you should set the viewport size before navigating to
        the page with `New Context` before opening the page itself.

        ``width`` <int> Sets the width size. **Required**

        ``height`` <int> Sets the heigth size. **Required**
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.SetViewportSize(
                Request().Viewport(height=height, width=width)
            )
            logger.info(response.log)

    @keyword(tags=["Setter", "BrowserControl"])
    def set_offline(self, offline: bool = True):
        """Toggles current Context's offline emulation.

        ``offline`` <bool> Toggles the offline mode. Set to False to switch back
        to online mode. Defaults to True.
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.SetOffline(Request().Bool(value=offline))
            logger.info(response.log)

    @keyword(tags=["Setter", "BrowserControl"])
    def set_geolocation(
        self, latitude: float, longitude: float, accuracy: Optional[float] = None
    ):
        """Updated the correct Context's geolocation.

        Latitude can be between -90 and 90 and longitude can be between -180 and 180.
        """
        geolocation_dict = {"latitude": latitude, "longitude": longitude}
        if accuracy:
            geolocation_dict["accuracy"] = accuracy
        geolocation = json.dumps(geolocation_dict)
        logger.info(geolocation)
        with self.playwright.grpc_channel() as stub:
            response = stub.SetGeolocation(Request().Json(body=geolocation))
            logger.info(response.log)

    @keyword(tags=["Setter", "BrowserControl"])
    def reload(self):
        """Reloads current active page."""
        with self.playwright.grpc_channel() as stub:
            response = stub.Reload(Request().Empty())
            logger.info(response.log)



# Copyright 2020-     Robot Framework Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from concurrent.futures._base import Future
from datetime import timedelta
from typing import TYPE_CHECKING, Set, Union

if TYPE_CHECKING:
    from ..browser import Browser


class LibraryComponent:
    def __init__(self, library: "Browser") -> None:
        """Base class exposing attributes from the common context.

        :param library: The library itself as a context object.
        """
        self.library = library

    @property
    def playwright(self):
        return self.library.playwright

    @property
    def timeout(self) -> float:
        return self.library.timeout

    @timeout.setter
    def timeout(self, value: float):
        self.library.timeout = value

    @property
    def retry_assertions_for(self) -> float:
        return self.library.retry_assertions_for

    @retry_assertions_for.setter
    def retry_assertions_for(self, value: float):
        self.library.retry_assertions_for = value

    @property
    def unresolved_promises(self):
        return self.library._unresolved_promises

    @unresolved_promises.setter
    def unresolved_promises(self, value: Set[Future]):
        self.library._unresolved_promises = value

    def get_timeout(self, timeout: Union[timedelta, None]) -> float:
        return self.library.get_timeout(timeout)

    def convert_timeout(self, timeout: Union[timedelta, float]) -> float:
        return self.library.convert_timeout(timeout)

    def millisecs_to_timestr(self, timeout: float) -> str:
        return self.library.millisecs_to_timestr(timeout)



# Copyright 2020-     Robot Framework Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
from datetime import datetime
from typing import List, Optional, Union

from robot.libraries.DateTime import convert_date  # type: ignore
from robot.utils import DotDict  # type: ignore
from robotlibcore import keyword  # type: ignore

from ..base import LibraryComponent
from ..generated.playwright_pb2 import Request
from ..utils import CookieSameSite, CookieType, locals_to_params, logger


class Cookie(LibraryComponent):
    @keyword(tags=["Getter", "PageContent"])
    def get_cookies(
        self, return_type: CookieType = CookieType.dictionary
    ) -> Union[List[DotDict], str]:
        """Returns cookies from the current active browser context.

        If ``return_type`` is ``dictionary`` or ``dict`` then keyword returns list of Robot Framework
        [https://robotframework.org/robotframework/latest/RobotFrameworkUserGuide.html#accessing-list-and-dictionary-items|dot dictionaries]
        The dictionary contains all possible key value pairs of the cookie. See `Get Cookie` keyword documentation
        about the dictionary keys and values.

        If ``return_type`` is ``string`` or ``str``, then keyword returns the cookie as a string in format:
        ``name1=value1; name2=value2; name3=value3``. The return value contains only ``name`` and ``value`` keys of the
        cookie.
        """
        response, cookies = self._get_cookies()
        if not response.log:
            logger.info("No cookies found.")
            return []
        else:
            logger.info(f"Found cookies: {response.log}")
        if return_type is CookieType.dictionary:
            return self._format_cookies_as_dot_dict(cookies)
        return self._format_cookies_as_string(cookies)

    def _get_cookies(self):
        with self.playwright.grpc_channel() as stub:
            response = stub.GetCookies(Request().Empty())
            return response, json.loads(response.json)

    def _format_cookies_as_string(self, cookies: List[dict]):
        pairs = []
        for cookie in cookies:
            pairs.append(self._cookie_as_string(cookie))
        return "; ".join(pairs)

    def _cookie_as_string(self, cookie: dict) -> str:
        return f'{cookie["name"]}={cookie["value"]}'

    def _format_cookies_as_dot_dict(self, cookies: List[dict]):
        as_list = []
        for cookie in cookies:
            as_list.append(self._cookie_as_dot_dict(cookie))
        return as_list

    def _cookie_as_dot_dict(self, cookie):
        dot_dict = DotDict()
        for key in cookie:
            if key == "expires":
                dot_dict[key] = datetime.fromtimestamp(cookie[key])
            else:
                dot_dict[key] = cookie[key]
        return dot_dict

    @keyword(tags=["Setter", "BrowserControl"])
    def add_cookie(
        self,
        name: str,
        value: str,
        url: Optional[str] = None,
        domain: Optional[str] = None,
        path: Optional[str] = None,
        expires: Optional[str] = None,
        httpOnly: Optional[bool] = None,
        secure: Optional[bool] = None,
        sameSite: Optional[CookieSameSite] = None,
    ):
        """Adds a cookie to currently active browser context.

        ``name`` <str> Name of the cookie. **Required**

        ``value`` <str> Given value for the cookie. **Required**

        ``url`` <str> Given url for the cookie. Defaults to None. Either ``url`` or ``domain`` / ``path`` pair must be set.

        ``domain`` <str> Given domain for the cookie. Defaults to None. Either ``url`` or ``domain`` / ``path`` pair must be set.

        ``path`` <str> Given path for the cookie. Defaults to None. Either ``url`` or ``domain`` / ``path`` pair must be set.

        ``expires`` <str> Given expiry for the cookie. Can be of date format or unix time. Supports
        the same formats as the [http://robotframework.org/robotframework/latest/libraries/DateTime.html|DateTime]
        library or an epoch timestamp.
        - example: 2027-09-28 16:21:35

        ``httpOnly`` <bool> Sets the httpOnly token.

        ``secure`` <bool> Sets the secure token.

        ``samesite`` < ``Strict`` | ``Lax`` | ``None`` > Sets the samesite mode.

        Example:
        | `Add Cookie`   foo   bar   http://address.com/path/to/site                                     # Using url argument.
        | `Add Cookie`   foo   bar   domain=example.com                path=/foo/bar                     # Using domain and url arguments.
        | `Add Cookie`   foo   bar   http://address.com/path/to/site   expiry=2027-09-28 16:21:35        # Expiry as timestamp.
        | `Add Cookie`   foo   bar   http://address.com/path/to/site   expiry=1822137695                 # Expiry as epoch seconds.
        """
        params = locals_to_params(locals())
        if expires:
            params["expires"] = self._expiry(expires)
        cookie_json = json.dumps(params)
        logger.debug(f"Adding cookie: {cookie_json}")
        with self.playwright.grpc_channel() as stub:
            response = stub.AddCookie(Request.Json(body=cookie_json))
            logger.info(response.log)

    def _expiry(self, expiry: str) -> int:
        try:
            return int(expiry)
        except ValueError:
            return int(convert_date(expiry, result_format="epoch"))

    @keyword(tags=["Setter", "BrowserControl"])
    def delete_all_cookies(self):
        """Deletes all cookies from the currently active browser context."""
        with self.playwright.grpc_channel() as stub:
            response = stub.DeleteAllCookies(Request.Empty())
        logger.info(response.log)

    @keyword
    def eat_all_cookies(self):
        """Eat all cookies for all easter."""
        self.delete_all_cookies()
        logger.info(
            """
        <iframe
        width="560" height="315"
        src="https://www.youtube.com/embed/I5e6ftNpGsU"
        frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture"
        allowfullscreen>
        </iframe>""",
            html=True,
        )
        logger.warn("Cookie monster ate all cookies!!")

    @keyword(tags=["Getter", "BrowserControl"])
    def get_cookie(
        self, cookie: str, return_type: CookieType = CookieType.dictionary
    ) -> Union[DotDict, str]:
        """Returns information of cookie with ``name`` as a Robot Framework dot dictionary or a string.

        ``cookie`` <str> Name of the cookie to be retrieved.

        If ``return_type`` is ``dictionary`` or ``dict`` then keyword returns a of Robot Framework
        [https://robotframework.org/robotframework/latest/RobotFrameworkUserGuide.html#accessing-list-and-dictionary-items|dot dictionary]
        The dictionary contains all possible key value pairs of the cookie. If ``return_type`` is ``string`` or ``str``,
        then keyword returns the cookie as a string in format: ``name1=value1``. The return value contains only
        ``name`` and ``value`` keys of the cookie.

        If no cookie is found with ``name`` keyword fails. The cookie dictionary contains
        details about the cookie. Keys available in the dictionary are documented in the table below.

        | *Value*  | *Explanation*                                                                              |
        | name     | The name of a cookie, mandatory.                                                           |
        | value    | Value of the cookie, mandatory.                                                            |
        | url      | Define the scope of the cookie, what URLs the cookies should be sent to.                   |
        | domain   | Specifies which hosts are allowed to receive the cookie.                                   |
        | path     | Indicates a URL path that must exist in the requested URL, for example `/`.                |
        | expiry   | Lifetime of a cookie. Returned as datatime object.                                         |
        | httpOnly | When true, the cookie is not accessible via JavaScript.                                    |
        | secure   | When true, the cookie is only used with HTTPS connections.                                 |
        | sameSite | Attribute lets servers require that a cookie shouldn't be sent with cross-origin requests. |

        See
        [https://github.com/microsoft/playwright/blob/master/docs/api.md#browsercontextaddcookiescookies|playwright documentation]
        for details about each attribute.

        Example:
        | ${cookie}=        Get Cookie              Foobar
        | Should Be Equal   ${cookie.value}         Tidii
        | Should Be Equal   ${cookie.expiry.year}   ${2020}
        """
        _, cookies = self._get_cookies()
        for cookie_dict in cookies:
            if cookie_dict["name"] == cookie:
                if return_type is CookieType.dictionary:
                    return self._cookie_as_dot_dict(cookie_dict)
                return self._cookie_as_string(cookie_dict)
        raise ValueError(f"Cookie with name {cookie} is not found.")



# Copyright 2020-     Robot Framework Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
from typing import Any, Optional

from robotlibcore import keyword  # type: ignore

from ..assertion_engine import verify_assertion, with_assertion_polling
from ..base import LibraryComponent
from ..generated.playwright_pb2 import Request
from ..utils import logger
from ..utils.data_types import AssertionOperator


class WebAppState(LibraryComponent):
    @keyword(name="localStorage get Item", tags=["PageContent", "Assertion", "Getter"])
    @with_assertion_polling
    def local_storage_get_item(
        self,
        key: str,
        assertion_operator: Optional[AssertionOperator] = None,
        assertion_expected: Any = None,
    ) -> Any:
        """
        Get saved data from the local storage.

        ``key`` <str> Named key of the item in the storage. **Required**

        See `Assertions` for further details for the assertion arguments. Defaults to None.
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.ExecuteJavascript(
                Request().JavascriptCode(script=f'window.localStorage.getItem("{key}")')
            )
            logger.info(response.log)
            return verify_assertion(
                json.loads(response.result),
                assertion_operator,
                assertion_expected,
                "localStorage ",
            )

    @keyword(name="localStorage set Item", tags=["Setter", "PageContent"])
    def local_storage_set_item(self, key: str, value: str):
        """
        Save data to the local storage.

        ``key`` <str> The name of the key under which it should be saved. **Required**

        ``value`` <str> The value which shall be saved as a string. **Required**
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.ExecuteJavascript(
                Request().JavascriptCode(
                    script=f'window.localStorage.setItem("{key}", "{value}")'
                )
            )
            logger.info(response.log)

    @keyword(name="localStorage remove Item", tags=["Setter", "PageContent"])
    def local_storage_remove_item(self, key: str):
        """
        Remove saved data with key from the local storage.

        ``key`` <str> Name of the item which shall be deleted. **Required**
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.ExecuteJavascript(
                Request().JavascriptCode(
                    script=f'window.localStorage.removeItem("{key}")'
                )
            )
            logger.info(response.log)

    @keyword(name="localStorage clear", tags=["Setter", "PageContent"])
    def local_storage_clear(self):
        """
        Remove all saved data from the local storage.
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.ExecuteJavascript(
                Request().JavascriptCode(script="window.localStorage.clear()")
            )
            logger.info(response.log)

    @keyword(
        name="sessionStorage get Item", tags=["PageContent", "Assertion", "Getter"]
    )
    @with_assertion_polling
    def session_storage_get_item(
        self,
        key: str,
        assertion_operator: Optional[AssertionOperator] = None,
        assertion_expected: Any = None,
    ) -> Any:
        """
        Get saved data from from session storage.

        ``key`` <str> Named key of the item in the storage. **Required**

        See `Assertions` for further details for the assertion arguments. Defaults to None.
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.ExecuteJavascript(
                Request().JavascriptCode(
                    script=f'window.sessionStorage.getItem("{key}")'
                )
            )
            logger.info(response.log)
            return verify_assertion(
                json.loads(response.result),
                assertion_operator,
                assertion_expected,
                "sessionStorage ",
            )

    @keyword(name="sessionStorage set Item", tags=["Setter", "PageContent"])
    def session_storage_set_item(self, key: str, value: str):
        """
        Save data to session storage.

        ``key`` <str> The name of the key under which it should be saved. **Required**

        ``value`` <str> The value which shall be saved as a string. **Required**
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.ExecuteJavascript(
                Request().JavascriptCode(
                    script=f'window.sessionStorage.setItem("{key}", "{value}")'
                )
            )
            logger.info(response.log)

    @keyword(name="sessionStorage remove Item", tags=["Setter", "PageContent"])
    def session_storage_remove_item(self, key: str):
        """
        Remove saved data with key from the session storage.

        ``key`` <str> Name of the item which shall be deleted. **Required**
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.ExecuteJavascript(
                Request().JavascriptCode(
                    script=f'window.sessionStorage.removeItem("{key}")'
                )
            )
            logger.info(response.log)

    @keyword(name="sessionStorage clear", tags=["Setter", "PageContent"])
    def session_storage_clear(self):
        """
        Remove all saved data from the session storage.
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.ExecuteJavascript(
                Request().JavascriptCode(script="window.sessionStorage.clear()")
            )
            logger.info(response.log)



# Copyright 2020-     Robot Framework Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from enum import Enum, auto

from typing_extensions import TypedDict

BoundingBox = TypedDict(
    "BoundingBox",
    {"x": float, "y": float, "width": float, "height": float},
    total=False,
)

Coordinates = TypedDict("Coordinates", {"x": float, "y": float}, total=False)

MouseOptionsDict = TypedDict(
    "MouseOptionsDict", {"x": float, "y": float, "options": dict}, total=False
)

ViewportDimensions = TypedDict("ViewportDimensions", {"width": int, "height": int})


class SelectionType(Enum):
    ACTIVE = auto()
    CURRENT = ACTIVE
    ALL = auto()
    ANY = ALL


class DialogAction(Enum):
    accept = auto()
    dismiss = auto()


class CookieType(Enum):
    dictionary = auto()
    dict = dictionary
    string = auto()
    str = string


CookieSameSite = Enum(
    "CookieSameSite", {"Strict": "Strict", "Lax": "Lax", "None": "None"}
)


class RequestMethod(Enum):
    HEAD = auto()
    GET = auto()
    POST = auto()
    PUT = auto()
    PATCH = auto()
    DELETE = auto()


class MouseButtonAction(Enum):
    click = auto()
    down = auto()
    up = auto()


class MouseButton(Enum):
    left = auto()
    middle = auto()
    right = auto()


class KeyAction(Enum):
    down = auto()
    up = auto()
    press = auto()


class KeyboardInputAction(Enum):
    insertText = auto()
    type = auto()


class KeyboardModifier(Enum):
    Alt = auto()
    Control = auto()
    Meta = auto()
    Shift = auto()


class SelectAttribute(Enum):
    value = auto()
    label = auto()
    text = label
    index = auto()


class SupportedBrowsers(Enum):
    chromium = auto()
    firefox = auto()
    webkit = auto()


ColorScheme = Enum("ColorScheme", ["dark", "light", "no-preference"])


ScrollBehavior = Enum("ScrollBehavior", ["auto", "smooth"])


class SizeFields(Enum):
    width = auto()
    height = auto()
    ALL = auto()


class AreaFields(Enum):
    top = auto()
    left = auto()
    bottom = auto()
    right = auto()
    ALL = auto()


class BoundingBoxFields(Enum):
    width = auto()
    height = auto()
    x = auto()
    y = auto()
    ALL = auto()


class AutoClosingLevel(Enum):
    SUITE = auto()
    TEST = auto()
    MANUAL = auto()


class ElementState(Enum):
    attached = auto()
    detached = auto()
    visible = auto()
    hidden = auto()
    enabled = auto()
    disabled = auto()
    editable = auto()
    readonly = auto()
    selected = auto()
    deselected = auto()
    focused = auto()
    defocused = auto()
    checked = auto()
    unchecked = auto()


AssertionOperator = Enum(
    "AssertionOperator",
    {
        "equal": "==",
        "==": "==",
        "should be": "==",
        "inequal": "!=",
        "!=": "!=",
        "should not be": "!=",
        "less than": "<",
        "<": "<",
        "greater than": ">",
        ">": ">",
        "<=": "<=",
        ">=": ">=",
        "contains": "*=",
        "*=": "*=",
        "starts": "^=",
        "^=": "^=",
        "should start with": "^=",
        "ends": "$=",
        "should end with": "$=",
        "$=": "$=",
        "matches": "$",
        "validate": "validate",
        "then": "then",
        "evaluate": "then",
    },
)



# Copyright 2020-     Robot Framework Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
from typing import Any, List, Optional, Union

from robotlibcore import keyword  # type: ignore

from ..assertion_engine import (
    bool_verify_assertion,
    dict_verify_assertion,
    float_str_verify_assertion,
    int_dict_verify_assertion,
    list_verify_assertion,
    verify_assertion,
    with_assertion_polling,
)
from ..base import LibraryComponent
from ..generated.playwright_pb2 import Request
from ..utils import exec_scroll_function, logger
from ..utils.data_types import (
    AreaFields,
    AssertionOperator,
    BoundingBoxFields,
    SelectAttribute,
    SizeFields,
)


class Getters(LibraryComponent):
    @keyword(tags=["Getter", "Assertion", "PageContent"])
    @with_assertion_polling
    def get_url(
        self,
        assertion_operator: Optional[AssertionOperator] = None,
        assertion_expected: Any = None,
    ) -> Any:
        """Returns the current URL.

        Optionally asserts that it matches the specified assertion.

        See `Assertions` for further details for the assertion arguments. Defaults to None.
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.GetUrl(Request().Empty())
            logger.debug(response.log)
            value = response.body
            return verify_assertion(
                value, assertion_operator, assertion_expected, "URL "
            )

    # @keyword(tags=["Getter", "Assertion", "BrowserControl"])
    # Not published as keyword due to missing of good docs.
    @with_assertion_polling
    def get_page_state(
        self,
        assertion_operator: Optional[AssertionOperator] = None,
        assertion_expected: Any = None,
    ) -> Any:
        """Returns page model state object as a dictionary.

        See `Assertions` for further details for the assertion arguments. Defaults to None.

        This must be given on the page to ``window.__SET_RFBROWSER_STATE__``

        For example:

        ``let mystate = {'login': true, 'name': 'George', 'age': 123};``

        ``window.__SET_RFBROWSER_STATE__ && window.__SET_RFBROWSER_STATE__(mystate);``
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.GetPageState(Request().Empty())
            logger.debug(response.log)
            value = json.loads(response.result)
            return verify_assertion(
                value, assertion_operator, assertion_expected, "State "
            )

    @keyword(tags=["Getter", "Assertion", "PageContent"])
    @with_assertion_polling
    def get_page_source(
        self,
        assertion_operator: Optional[AssertionOperator] = None,
        assertion_expected: Any = None,
    ) -> Any:
        """Gets pages HTML source as a string.

        Optionally does a string assertion.

        See `Assertions` for further details for the assertion arguments. Defaults to None.
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.GetPageSource(Request().Empty())
            logger.debug(response.log)
            value = json.loads(response.body)
            return verify_assertion(
                value, assertion_operator, assertion_expected, "HTML: "
            )

    @keyword(tags=["Getter", "Assertion", "PageContent"])
    @with_assertion_polling
    def get_title(
        self,
        assertion_operator: Optional[AssertionOperator] = None,
        assertion_expected: Any = None,
    ) -> Any:
        """Returns the title of the current page.

        Optionally asserts that it matches the specified assertion.

        See `Assertions` for further details for the assertion arguments. Defaults to None.
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.GetTitle(Request().Empty())
            logger.debug(response.log)
            value = response.body
            return verify_assertion(
                value, assertion_operator, assertion_expected, "Title "
            )

    @keyword(tags=["Getter", "Assertion", "PageContent"])
    def get_text(
        self,
        selector: str,
        assertion_operator: Optional[AssertionOperator] = None,
        assertion_expected: Any = None,
    ) -> Any:
        """Returns text attribute of the element found by ``selector``.
        See the `Finding elements` section for details about the selectors.

        Optionally asserts that the text matches the specified assertion.

        See `Assertions` for further details for the assertion arguments. Defaults to None.
        """
        return self.get_property(
            selector, "innerText", assertion_operator, assertion_expected
        )

    @keyword(tags=["Getter", "Assertion", "PageContent"])
    @with_assertion_polling
    def get_property(
        self,
        selector: str,
        property: str,
        assertion_operator: Optional[AssertionOperator] = None,
        assertion_expected: Any = None,
    ) -> Any:
        """Returns the ``property`` of the element found by ``selector``.

        Optionally asserts that the property value matches the specified
        assertion.

        ``selector`` <str> Selector from which the info is to be retrieved. **Required**
        See the `Finding elements` section for details about the selectors.

        ``property`` <str> Requested property name. **Required**

        If ``assertion_operator`` is set and property is not found, ``value`` is ``None``
        and Keyword does not fail. See `Get Attribute` for examples.

        See `Assertions` for further details for the assertion arguments. Defaults to None.
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.GetDomProperty(
                Request().ElementProperty(selector=selector, property=property)
            )
            logger.debug(response.log)
            if response.body:
                value = json.loads(response.body)
            elif assertion_operator is not None:
                value = None
            else:
                raise AttributeError(f"Property '{property}' not found!")
            return verify_assertion(
                value, assertion_operator, assertion_expected, f"Property {property}"
            )

    @keyword(tags=["Getter", "Assertion", "PageContent"])
    @with_assertion_polling
    def get_attribute(
        self,
        selector: str,
        attribute: str,
        assertion_operator: Optional[AssertionOperator] = None,
        assertion_expected: Any = None,
    ) -> Any:
        """Returns the HTML ``attribute`` of the element found by ``selector``.

        Optionally asserts that the attribute value matches the specified
        assertion.

        ``selector`` <str> Selector from which the info is to be retrieved. **Required**
        See the `Finding elements` section for details about the selectors.

        ``attribute`` <str> Requested attribute name. **Required**

        When a attribute is selected that is not present and no assertion operator is set,
        the keyword fails. If an assertion operator is set and the attribute is not present,
        the returned value is ``None``.
        This can be used to assert check the presents or the absents of an attribute.

        Example Element:
        | <button class="login button active" id="enabled_button" something>Login</button>

        Example Code:
        | Get Attribute   id=enabled_button    disabled                   # FAIL => "Attribute 'disabled' not found!"
        | Get Attribute   id=enabled_button    disabled     ==    None     # PASS => returns: None
        | Get Attribute   id=enabled_button    something    evaluate    value is not None    # PASS =>  returns: True
        | Get Attribute   id=enabled_button    disabled     evaluate    value is None        # PASS =>  returns: True


        See `Assertions` for further details for the assertion arguments. Defaults to None.
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.GetElementAttribute(
                Request().ElementProperty(selector=selector, property=attribute)
            )
            logger.debug(response.log)
            value = json.loads(response.body)
            if assertion_operator is None and value is None:
                raise AttributeError(f"Attribute '{attribute}' not found!")
            logger.debug(f"Attribute is: {value}")
            return verify_assertion(
                value, assertion_operator, assertion_expected, f"Attribute {selector}"
            )

    @keyword(tags=["Getter", "Assertion", "PageContent"])
    @with_assertion_polling
    def get_attribute_names(
        self,
        selector: str,
        assertion_operator: Optional[AssertionOperator] = None,
        *assertion_expected,
    ) -> Any:
        """Returns all HTML attribute names of an element as a list.

        Optionally asserts that these match the specified assertion.

        ``selector`` <str> Selector from which the info is to be retrieved. **Required**
        See the `Finding elements` section for details about the selectors.

        ``assertion_operator`` <AssertionOperator> See `Assertions` for further details. Defaults to None.

        Available assertions:
        - ``==`` and ``!=`` can work with multiple values
        - ``contains``/``*=`` only accepts one single expected value

        Other operators are not allowed.
        """
        attribute_names = self.library.execute_javascript(
            "(element) => element.getAttributeNames()", selector
        )
        expected = list(assertion_expected)
        return list_verify_assertion(
            attribute_names, assertion_operator, expected, "Attribute names"
        )

    @keyword(tags=["Getter", "Assertion", "PageContent"])
    @with_assertion_polling
    def get_classes(
        self,
        selector: str,
        assertion_operator: Optional[AssertionOperator] = None,
        *assertion_expected,
    ) -> Any:
        """Returns all classes of an element as a list.

        Optionally asserts that the value matches the specified assertion.

        ``selector`` <str> Selector from which the info is to be retrieved. **Required**
        See the `Finding elements` section for details about the selectors.

        ``assertion_operator`` <AssertionOperator> See `Assertions` for further details. Defaults to None.

        Available assertions:
        - ``==`` and ``!=`` can work with multiple values
        - ``contains``/``*=`` only accepts one single expected value

        Other operators are not allowed.
        """
        class_dict = self.get_property(selector, "classList")
        expected = list(assertion_expected)
        return list_verify_assertion(
            list(class_dict.values()),
            assertion_operator,
            expected,
            f"Classes of {selector}",
        )

    @keyword(tags=["Getter", "Assertion", "PageContent"])
    @with_assertion_polling
    def get_textfield_value(
        self,
        selector: str,
        assertion_operator: Optional[AssertionOperator] = None,
        assertion_expected: Any = None,
    ) -> Any:
        """Returns value of the textfield found by ``selector``.

        Optionally asserts that the value matches the specified assertion.

        ``selector`` <str> Selector from which the info is to be retrieved. **Required**
        See the `Finding elements` section for details about the selectors.

        See `Assertions` for further details for the assertion arguments. Defaults to None.
        """
        return verify_assertion(
            self.get_property(selector, "value"),
            assertion_operator,
            assertion_expected,
            f"Value {selector}",
        )

    @keyword(tags=["Getter", "Assertion", "PageContent"])
    @with_assertion_polling
    def get_selected_options(
        self,
        selector: str,
        option_attribute: SelectAttribute = SelectAttribute.label,
        assertion_operator: Optional[AssertionOperator] = None,
        *assertion_expected,
    ) -> Any:
        """Returns the specified attribute of selected options of the ``select`` element.

        Optionally asserts that these match the specified assertion.

        ``selector`` <str> Selector from which the info is to be retrieved. **Required**
        See the `Finding elements` section for details about the selectors.

        ``option_attribute`` <SelectAttribute.label> Which attribute shall be returned/verified.
        Allowed values are ``< ``value`` | ``label`` | ``text`` | ``index`` >``. Defaults to label.

        ``assertion_operator`` <AssertionOperator> See `Assertions` for further details. Defaults to None.

        - ``==`` and ``!=`` can work with multiple values
        - ``contains``/``*=`` only accepts one single expected value

        Other operators are not allowed.

        Example:

        | `Select Options By`      label                    //select[2]    Email      Mobile
        | ${selected_list}         `Get Selected Options`   //select[2]                                         # getter
        | `Get Selected Options`   //select[2]              label          `==`       Mobile             Mail   #assertion content
        | `Select Options By`      label                    select#names   2          4
        | `Get Selected Options`   select#names             index          `==`       2                  4      #assertion index
        | `Get Selected Options`   select#names             label          *=         Mikko                     #assertion contain
        | `Get Selected Options`   select#names             label          validate   len(value) == 3           #assertion length

        """
        with self.playwright.grpc_channel() as stub:
            response = stub.GetSelectContent(
                Request().ElementSelector(selector=selector)
            )
            logger.info(response)
            expected = list(assertion_expected)
            selected: Union[List[int], List[str]]
            if option_attribute is SelectAttribute.value:
                selected = [sel.value for sel in response.entry if sel.selected]
            elif option_attribute is SelectAttribute.label:
                selected = [sel.label for sel in response.entry if sel.selected]
            elif option_attribute is SelectAttribute.index:
                selected = [
                    index for index, sel in enumerate(response.entry) if sel.selected
                ]
                expected = [int(exp) for exp in expected]

            return list_verify_assertion(
                selected,
                assertion_operator,
                expected,
                "Selected Options:",
            )

    @keyword(tags=["Getter", "Assertion", "PageContent"])
    @with_assertion_polling
    def get_checkbox_state(
        self,
        selector: str,
        assertion_operator: Optional[AssertionOperator] = None,
        expected_state: Union[bool, str] = "Unchecked",
    ) -> bool:
        """Returns the state of the checkbox found by ``selector``.

        ``selector`` <str> Selector which shall be examined.
        See the `Finding elements` section for details about the selectors.

        Optionally asserts that the state matches the specified assertion.

        ``assertion_operator`` <AssertionOperator> See `Assertions` for further details. Defaults to None.

        - ``==`` and ``!=`` and equivalent are allowed on boolean values
        - other operators are not accepted.

        ``expected_state`` <str> boolean value of expected state.
        Strings are interpreted as booleans.
        All strings are ``${True}`` except of the
        following `FALSE, NO, OFF, 0, UNCHECKED, NONE, ${EMPTY}``.
        (case-insensitive). Defaults to unchecked

        - ``checked`` => ``True``
        - ``unchecked`` => ``False``

        """
        with self.playwright.grpc_channel() as stub:
            response = stub.GetBoolProperty(
                Request().ElementProperty(selector=selector, property="checked")
            )
            logger.info(response.log)
            value: bool = response.body
            logger.info(f"Checkbox is {'checked' if value else 'unchecked'}")
            return bool_verify_assertion(
                value, assertion_operator, expected_state, f"Checkbox {selector} is"
            )

    @keyword(tags=["Getter", "Assertion", "PageContent"])
    @with_assertion_polling
    def get_element_count(
        self,
        selector: str,
        assertion_operator: Optional[AssertionOperator] = None,
        expected_value: Union[int, str] = 0,
    ) -> Any:
        """Returns the count of elements found with ``selector``.

        Optionally asserts that the count matches the specified assertion.

        ``selector`` <str> Selector which shall be counted.
        See the `Finding elements` section for details about the selectors.

        ``assertion_operator`` <AssertionOperator> See `Assertions` for further details. Defaults to None.

        ``expected_value`` <str|int> Expected value for the counting
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.GetElementCount(
                Request().ElementSelector(selector=selector)
            )
            count = response.body
            return float_str_verify_assertion(
                int(count),
                assertion_operator,
                expected_value,
                f"Element count for selector `{selector}` is",
            )

    @keyword(tags=["Getter", "Assertion", "BrowserControl"])
    @with_assertion_polling
    def get_viewport_size(
        self,
        key: SizeFields = SizeFields.ALL,
        assertion_operator: Optional[AssertionOperator] = None,
        assertion_expected: Any = None,
    ) -> Any:
        """Returns the current viewport dimensions.

        Optionally asserts that the count matches the specified assertion.

        ``key`` < ``width`` | ``height`` | ``ALL`` > Optionally filters the returned values.
        If keys is set to ``ALL``(default) it will return the viewport size as dictionary,
        otherwise it will just return the single value selected by the key.
        Note: If a single value is retrieved, an assertion does *not* need a ``validate``
        combined with a cast of ``value``.

        See `Assertions` for further details for the assertion arguments. Defaults to None.

        Example:
        | Get Viewport Size    ALL    ==    {'width':1280, 'height':720}
        | Get Viewport Size    width    >=    1200

        """
        with self.playwright.grpc_channel() as stub:
            response = stub.GetViewportSize(Request().Empty())
            logger.info(response.log)
            parsed = json.loads(response.json)
            logger.debug(parsed)
            if key == SizeFields.ALL:
                return int_dict_verify_assertion(
                    parsed, assertion_operator, assertion_expected, "Viewport size is"
                )
            else:
                logger.info(f"Value of '{key}'': {parsed[key.name]}")
                return float_str_verify_assertion(
                    parsed[key.name],
                    assertion_operator,
                    assertion_expected,
                    f"{key} is ",
                )

    @keyword(tags=["Getter", "PageContent"])
    def get_element(self, selector: str) -> str:
        """Returns a reference to a Playwright element handle.

        The reference can be used in subsequent selectors.

        ``selector`` <str> Selector from which shall be retrieved **Required**.
        See the `Finding elements` section for details about the selectors.
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.GetElement(Request().ElementSelector(selector=selector))
            return response.body

    @keyword(tags=["Getter", "PageContent"])
    def get_elements(self, selector: str) -> List[str]:
        """Returns a reference to playwright element handle for all matched elements by ``selector``.

        ``selector`` <str> Selector from which shall be retrieved. **Required**
        See the `Finding elements` section for details about the selectors.
        """
        try:
            with self.playwright.grpc_channel() as stub:
                response = stub.GetElements(
                    Request().ElementSelector(selector=selector)
                )
                return json.loads(response.json)
        except AssertionError as error:
            if "Could not find element with" in str(error):
                return []
            raise error

    @keyword(tags=["Getter", "Assertion", "PageContent"])
    @with_assertion_polling
    def get_style(
        self,
        selector: str,
        key: str = "ALL",
        assertion_operator: Optional[AssertionOperator] = None,
        assertion_expected: Any = None,
    ) -> Any:
        """Gets the computed style properties of the element selected by ``selector``.

        Optionally matches with any sequence assertion operator.

        ``selector`` <str> Selector from which the style shall be retrieved. **Required**
        See the `Finding elements` section for details about the selectors.

        ``key`` <str> Key of the requested CSS property. Retrieves "ALL" styles by default.

        See `Assertions` for further details for the assertion arguments. Defaults to None.
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.GetStyle(Request().ElementSelector(selector=selector))
            parsed = json.loads(response.json)

            if key == "ALL":
                return dict_verify_assertion(
                    parsed, assertion_operator, assertion_expected, "Computed style is"
                )
            else:
                item = parsed.get(key, "NOT_FOUND")
                logger.info(f"Value of key: {key}")
                logger.info(f"Value of selected property: {item}")
                return verify_assertion(
                    item,
                    assertion_operator,
                    assertion_expected,
                    f"Style value for {key} is ",
                )

    @keyword(name="Get BoundingBox", tags=["Getter", "Assertion", "PageContent"])
    def get_boundingbox(
        self,
        selector: str,
        key: BoundingBoxFields = BoundingBoxFields.ALL,
        assertion_operator: Optional[AssertionOperator] = None,
        assertion_expected: Any = None,
    ) -> Any:
        """Gets elements size and location as an object ``{x: float, y: float, width: float, height: float}``.

        ``selector`` <str> Selector from which shall be retrieved. **Required**
        See the `Finding elements` section for details about the selectors.

        ``key`` < ``x`` | ``y`` | ``width`` | ``height`` | ``ALL`` > Optionally filters the returned values.
        If keys is set to ``ALL``(default) it will return the BoundingBox as Dictionary,
        otherwise it will just return the single value selected by the key.
        Note: If a single value is retrieved, an assertion does *not* need a ``validate``
        combined with a cast of ``value``.

        See `Assertions` for further details for the assertion arguments. Defaults to None.

        Example use:
        | ${bounding_box}=    Get BoundingBox    id=element                 # unfiltered
        | Log                 ${bounding_box}                               # {'x': 559.09375, 'y': 75.5, 'width': 188.796875, 'height': 18}
        | ${x}=               Get BoundingBox    id=element    x            # filtered
        | Log                 X: ${x}                                       # X: 559.09375
        | # Assertions:
        | Get BoundingBox     id=element         width         >    180
        | Get BoundingBox     id=element         ALL           validate    value['x'] > value['y']*2
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.GetBoundingBox(Request.ElementSelector(selector=selector))
            parsed = json.loads(response.json)
            logger.debug(parsed)
            if key == BoundingBoxFields.ALL:
                return int_dict_verify_assertion(
                    parsed, assertion_operator, assertion_expected, "BoundingBox is"
                )
            else:
                logger.info(f"Value of '{key}'': {parsed[key.name]}")
                return float_str_verify_assertion(
                    parsed[key.name],
                    assertion_operator,
                    assertion_expected,
                    f"BoundingBox {key.name} is ",
                )

    @keyword(tags=["Getter", "Assertion", "PageContent"])
    def get_scroll_size(
        self,
        selector: Optional[str] = None,
        key: SizeFields = SizeFields.ALL,
        assertion_operator: Optional[AssertionOperator] = None,
        assertion_expected: Any = None,
    ) -> Any:
        """Gets elements or pages scrollable size as object ``{width: float, height: float}``.

        ``selector`` <str> Optional selector from which shall be retrieved.
        If no selector is given the scroll size of the page itself is used.
        See the `Finding elements` section for details about the selectors.

        ``key`` < ``width`` | ``height`` | ``ALL`` > Optionally filters the returned values.
        If keys is set to ``ALL``(default) it will return the scroll size as dictionary,
        otherwise it will just return the single value selected by the key.

        See `Assertions` for further details for the assertion arguments. Defaults to None.

        See `Get BoundingBox` for more similar examples.

        Example use:
        | ${height}=         Get Scroll Size    height                          # filtered page by height
        | Log                Width: ${height}                                   # Height: 58425
        | ${scroll_size}=    Get Scroll Size    id=keyword-shortcuts-container  # unfiltered element
        | Log                ${scroll_size}                                     # {'width': 253, 'height': 3036}
        """
        scroll_size = dict()
        scroll_size["width"] = exec_scroll_function(self, "scrollWidth", selector)
        scroll_size["height"] = exec_scroll_function(self, "scrollHeight", selector)
        if key == SizeFields.ALL:
            return int_dict_verify_assertion(
                scroll_size,
                assertion_operator,
                assertion_expected,
                "Scroll size is",
            )
        else:
            logger.info(f"Value of '{key}'': {scroll_size[key.name]}")
            return float_str_verify_assertion(
                scroll_size[key.name],
                assertion_operator,
                assertion_expected,
                f"Scroll {key.name} is ",
            )

    @keyword(tags=["Getter", "Assertion", "PageContent"])
    def get_scroll_position(
        self,
        selector: Optional[str] = None,
        key: AreaFields = AreaFields.ALL,
        assertion_operator: Optional[AssertionOperator] = None,
        assertion_expected: Any = None,
    ) -> Any:
        """Gets elements or pages current scroll position as object ``{top: float, left: float, bottom: float, right: float}``.

        It describes the rectangle which is visible of the scrollable content of that element.
        all values are measured from position {top: 0, left: 0}.

        ``top`` uses js function scrollTop, ``left`` uses scrollLeft and
        ``bottom`` and ``right`` are calculated with the client size.

        ``selector`` <str> Optional selector from which shall be retrieved.
        If no selector is given the client size of the page itself is used (``document.scrollingElement``).
        See the `Finding elements` section for details about the selectors.

        ``key`` < ``top`` | ``left`` | ``bottom`` | ``right`` | ``ALL`` > Optionally filters the returned values.
        If keys is set to ``ALL``(default) it will return the scroll position as dictionary,
        otherwise it will just return the single value selected by the key.

        See `Assertions` for further details for the assertion arguments. Defaults to None.

        See `Get BoundingBox` or `Get Scroll Size` for examples.
        """
        scroll_position = dict()
        scroll_position["top"] = exec_scroll_function(self, "scrollTop", selector)
        scroll_position["left"] = exec_scroll_function(self, "scrollLeft", selector)
        client_size = self.get_client_size(selector)
        scroll_position["bottom"] = scroll_position["top"] + client_size["height"]
        scroll_position["right"] = scroll_position["left"] + client_size["width"]
        if key == AreaFields.ALL:
            return int_dict_verify_assertion(
                scroll_position,
                assertion_operator,
                assertion_expected,
                "Scroll position is",
            )
        else:
            logger.info(f"Value of '{key}'': {scroll_position[key.name]}")
            return float_str_verify_assertion(
                scroll_position[key.name],
                assertion_operator,
                assertion_expected,
                f"Scroll position {key.name} is ",
            )

    @keyword(tags=["Getter", "Assertion", "PageContent"])
    def get_client_size(
        self,
        selector: Optional[str] = None,
        key: SizeFields = SizeFields.ALL,
        assertion_operator: Optional[AssertionOperator] = None,
        assertion_expected: Any = None,
    ) -> Any:
        """Gets elements or pages client size (``clientHeight``, ``clientWidth``) as object {width: float, height: float}.

        ``selector`` <str> Optional selector from which shall be retrieved.
        If no selector is given the client size of the page itself is used (``document.scrollingElement``).
        See the `Finding elements` section for details about the selectors.

        ``key`` < ``width`` | ``height`` | ``ALL`` > Optionally filters the returned values.
        If keys is set to ``ALL``(default) it will return the scroll size as dictionary,
        otherwise it will just return the single value selected by the key.

        See `Assertions` for further details for the assertion arguments. Defaults to None.

        See `Get BoundingBox` or `Get Scroll Size` for examples.
        """
        client_size = dict()
        client_size["width"] = exec_scroll_function(self, "clientWidth", selector)
        client_size["height"] = exec_scroll_function(self, "clientHeight", selector)
        if key == SizeFields.ALL:
            return int_dict_verify_assertion(
                client_size,
                assertion_operator,
                assertion_expected,
                "Client size is",
            )
        else:
            logger.info(f"Value of '{key}'': {client_size[key.name]}")
            return float_str_verify_assertion(
                client_size[key.name],
                assertion_operator,
                assertion_expected,
                f"Client {key.name} is ",
            )
